import os
import plotly.express as px
import plotly.graph_objects as go
import pandas as pd
from pathlib import Path
from textblob import TextBlob
from vaderSentiment.vaderSentiment import SentimentIntensityAnalyzer
from tqdm import tqdm

analyser = SentimentIntensityAnalyzer()
class Sentimentanalyze:
    def __init__(self,clean_df, column_name,date_column,vader):
        self.clean_df=clean_df
        self.column_name=column_name
        self.date_column=date_column
        self.vader=vader



    @staticmethod
    def _prediction_textblob(text):
        return 0 if (TextBlob(text).sentiment.polarity <= 0)else 1

    @staticmethod
    def _prediction_vader(text):
        sentiment_score = analyser.polarity_scores(text)

        return 1 if (sentiment_score['compound']  >= 0.05 ) else 0

    def showing_sentiment(self):
        df1 = self.clean_df.copy()
        if self.vader== False:
            tqdm.pandas(desc="Applying sentiment scores using TextBlob classifier")
            df1['sentiment'] = df1['reviewText'].progress_apply(self._prediction_textblob)



        else:
                tqdm.pandas(desc="Applying sentiment scores using Vader classifier")

                df1['sentiment']=df1['reviewText'].progress_apply(self._prediction_vader)

        fig1 = go.Figure(data=[go.Pie(labels=['positive', 'negative'],name="overall sentiments", values=df1['sentiment'].value_counts())],layout=go.Layout(
        title=go.layout.Title(text="Overall Sentiment Analysis")
    ))

        fig1.update_traces(marker=dict(colors=px.colors.sequential.Agsunset , line=dict(color='#000000', width=2)))

        if self.date_column != None:
            try:

                df_week = df1.groupby([pd.Grouper(key=str(self.date_column), freq='W'), 'sentiment']).size().unstack()
                data_week = pd.DataFrame(
                    {'date': (df_week.index), 'negative': (df_week[0].values), 'positive': (df_week[1].values)})

                df_month = df1.groupby([pd.Grouper(key=str(self.date_column), freq='M'), 'sentiment']).size().unstack()
                data_month = pd.DataFrame(
                    {'date': (df_month.index), 'negative': (df_month[0].values), 'positive': (df_month[1].values)})

                weekly_plot_negative=go.Scatter(x=data_week['date'], y=data_week['negative'],
                                         mode='lines+markers',
                                         name='number of Negative reviews per week')
                weekly_plot_positive=go.Scatter(x=data_week['date'], y=data_week['positive'],
                                         mode='lines+markers',
                                         name='number of Positive reviews per week')
                data_week=[weekly_plot_negative,weekly_plot_positive]

                layout_month={

                    'xaxis':dict(
                        tickformat="%Y-%B",
                        showline=True,
                        showgrid=False,
                        showticklabels=True,
                        linecolor='rgb(204, 204, 204)',
                        linewidth=2,
                        ticks='outside',
                        tickfont=dict(
                            family='Arial',
                            size=12,
                            color='rgb(82, 82, 82)',
                        ),
                    ),
                    'yaxis':dict(
                        showgrid=False,
                        zeroline=False,
                        showline=False
                    ),
                    'title':dict(text="Monthly sentiment analysis"),

                    'xaxis_title' : dict(text='Year-Month'),
                    'yaxis_title' :dict(text='Number of reviews'),
                    'autosize':True,
                    'margin':dict(
                        autoexpand=False,
                        l=100,
                        r=20,
                        t=110,
                    )}
                layout_week = {
                    'xaxis': dict(
                        tickformat="%Y-%B-%w",
                        showline=True,
                        showgrid=False,
                        showticklabels=True,
                        linecolor='rgb(204, 204, 204)',
                        linewidth=2,
                        ticks='outside',
                        tickfont=dict(
                            family='Arial',
                            size=12,
                            color='rgb(82, 82, 82)',
                        ),
                    ),
                    'yaxis': dict(
                        showgrid=False,
                        zeroline=False,
                        showline=False
                    ),
                    'autosize': True,
                    'title': dict(text="Monthly sentiment analysis"),

                    'xaxis_title' : dict(text='Year-Month-week'),
                    'yaxis_title' :dict(text='Number of reviews'),
                    'margin': dict(
                        autoexpand=False,
                        l=100,
                        r=20,
                        t=110,
                    )}

                fig2 = go.Figure(data=data_week,layout=layout_week)

                monthly_plot_negative= go.Scatter(x=data_month['date'], y=data_month['negative'],
                                  mode='lines+markers',
                                  name='number of Negative reviews per week')
                monthly_plot_positive = go.Scatter(x=data_month['date'], y=data_month['positive'],
                                   mode='lines+markers',
                                   name='number of Positive reviews per week')
                data_month = [monthly_plot_negative, monthly_plot_positive]
                fig3 = go.Figure(data=data_month, layout=layout_month)
                self.figures_to_html([fig1, fig2, fig3,self.fig_aspect])

            except:
                    print('Date column is invalid')

        else:

                    self.figures_to_html([fig1,self.fig_aspect])

    @staticmethod
    def figures_to_html(figs, filename="dashboard.html"):
            dashboard = open(filename, 'w')
            dashboard.write("<html><head></head><body>" + "\n")
            for fig in figs:

                inner_html = fig.to_html().split('<body>')[1].split('</body>')[0]
                dashboard.write(inner_html)
            dashboard.write("</body></html>" + "\n")










from insightsearch.data_clean import *
from insightsearch.dataframe_checking import *
from pathlib import Path

class Analysis(DataframeClean):
    def __init__(
        self, df, column_name , date_column= None, vader=False ):

        self.df = df
        self.column_name = column_name
        self.date_column = date_column
        self.vader=vader
        DFvalid.__init__(self, self.df, self.column_name, self.date_column,self.vader)
        self._clean()
    def review_analyze(self):
        self.showing_aspect()
        self.showing_sentiment()







import pandas as pd
from insightsearch.data_clean import  DataframeClean


class DFvalid:
    def __init__(self, df, column_name, date_column,vader):
        self.df = pd.read_csv(df)
        self.column_name = str(column_name)
        self.date_column = date_column
        self.vader=vader
        """
        parm df :df is the dataframe that contains review column , df doenot need all rows should be string. Datatypes other than str will be removed
        param column_name = Name of the column that contains review text 

        """
        DataframeClean.__init__(self, self.df, self.column_name, self.date_column,self.vader)
        # chechking df is a pd.DataFrame
        if self.df is not None:
            if isinstance(self.df, pd.DataFrame) != True:
                raise AttributeError("Data you were uploded is not a dataframe")

            if self.column_name is not None:
                if isinstance(self.column_name, str) != True:
                    raise AttributeError("The dataframe name is not string ")
                if self.column_name not in self.df.columns:
                    raise AttributeError(
                        "Your mentioned name in column_name doesnot exists in your uploaded dataframe"
                    )
            else:
                print("Try to mention a column name where your reviews stored")

            if self.date_column != None:
                if isinstance(self.date_column, str) != True:
                    raise AttributeError("The dataframe name is not string ")
                if self.date_column not in self.df.columns:
                    raise AttributeError(
                        "Your mentioned name in date column does not exists in your uploaded dataframe"
                    )
            else:
                pass
        else:
            print("There is no data frame")



from nltk.corpus import stopwords
from collections import Counter
import plotly.graph_objects as go
from plotly.subplots import make_subplots
from textblob import TextBlob
from insightsearch.sentiment_analyze import *
from tqdm import tqdm
tqdm.pandas()



class Aspectanalyze(Sentimentanalyze):
    def __init__(self, df, column_name):
        self.stop_words = set(stopwords.words("english"))
        self.df = df

        self.column_name = column_name
        self.lines = [l for l in self.df["reviewText"]]

    def _aspect_list(self, line, candidate_aspects, aspect_opinion_dict):
        blob = TextBlob(line)

        if len(candidate_aspects) > 0:
            for a in candidate_aspects:
                first_w_index = blob.words.index(a.split()[0])

                last_w_index = blob.words.index(a.split()[-1])

                # chechking whether word_index-1 is adj if yes will add to list
                pref_words = []
                if self.index_valid(first_w_index - 1 , blob.tags):
                    if blob.tags[first_w_index - 1][1] in [
                       "RB",
                        "RBR",
                        "RBS",
                        "JJ",
                        "JJR",
                        "JJS",
                    ] and blob.tags[first_w_index - 1][0].lower().strip() not in [
                        "very",
                        "really","much"
                    ]:

                        if self.index_valid(first_w_index - 2, blob.tags) and blob.tags[first_w_index - 2][0].lower().strip() in [
                               "not"] :

                            pref_words.append(
                               "not" + str(blob.tags[first_w_index - 1][0].lower().strip())
                            )

                        else:
                            pref_words.append(
                               blob.tags[first_w_index - 1][0].lower().strip()
                            )





                if len(pref_words) > 0:
                    if self.index_valid(first_w_index - 2, blob.tags):
                        if blob.tags[first_w_index - 2][1] in [
                            "RB",
                            "RBR",
                            "RBS",
                            "JJ",
                            "JJR",
                            "JJS",
                        ] and blob.tags[first_w_index - 2][0].lower().strip() not in [
                            "very",
                            "really","much","and"
                        ]:
                            if self.index_valid(first_w_index - 3, blob.tags) and blob.tags[first_w_index - 3][0].lower().strip() in [
                               "not"] :

                                pref_words.append(
                               "not" + str(blob.tags[first_w_index - 2][0].lower().strip())
                            )

                            else:
                                pref_words.append(
                               blob.tags[first_w_index - 2][0].lower().strip()

                            )
                    if self.index_valid(first_w_index - 2, blob.tags):
                        if blob.tags[first_w_index - 2][1] in [
                            "RB",
                            "RBR",
                            "RBS",
                            "JJ",
                            "JJR",
                            "JJS",
                        ] and blob.tags[first_w_index - 2][0].lower().strip() in [
                            "and"
                        ]:
                            pref_words.append(
                                blob.tags[first_w_index - 3][0].lower().strip()

                            )

                    if self.index_valid(first_w_index - 3, blob.tags):
                        if blob.tags[first_w_index - 3][1] in [
                            "RB",
                            "RBR",
                            "RBS",
                            "JJ",
                            "JJR",
                            "JJS",
                        ] and blob.tags[first_w_index - 3][0].lower().strip() not in [
                            "very",
                            "really","much","and"
                        ]:
                            pref_words.append(
                                blob.tags[first_w_index - 3][0].lower().strip()
                            )

                attr = "".join(pref_words)
                if str(a.strip()) in list(aspect_opinion_dict.keys()):

                    aspect_opinion_dict[a] = " ".join(
                        [aspect_opinion_dict[a] + " " + attr]
                    ).strip()
                else:
                    aspect_opinion_dict[str(a.strip())] = attr

                after_words = []

                if self.index_valid(first_w_index + 1, blob.tags) and blob.words[first_w_index + 1].lower() in ["am", "is", "are", "was", "were"]:
                    if self.index_valid(first_w_index + 2, blob.tags):

                        if blob.tags[last_w_index + 2][1] in [
                            "RB",
                            "RBR",
                            "RBS",
                            "JJ",
                            "JJR",
                            "JJS",
                        ] and blob.tags[last_w_index + 2][0].lower().strip() not in [
                            "very",
                            "really","always"
                        ] :
                            after_words.append(
                                blob.tags[last_w_index + 2][0].lower().strip()
                            )

                    if self.index_valid(first_w_index + 3, blob.tags):

                        if blob.tags[last_w_index + 3][1] in [
                            "RB",
                            "RBR",
                            "RBS",
                            "JJ",
                            "JJR",
                            "JJS",
                        ] and blob.tags[last_w_index + 3][0].lower().strip() in [
                            "and"

                        ] :
                            if self.index_valid(first_w_index + 4, blob.tags):

                                after_words.append(
                                    blob.tags[last_w_index + 4][0].lower().strip()
                            )


                    if len(after_words) > 0:
                        if self.index_valid(first_w_index + 3, blob.tags):
                            if blob.tags[last_w_index + 3][1] in [
                                "RB",
                                "RBR",
                                "RBS",
                                "JJ",
                                "JJR",
                                "JJS",
                            ] and blob.tags[first_w_index + 3][0].lower().strip() not in [
                                "very",
                                "really","much","and"
                            ]:
                                after_words.append(
                                    blob.tags[last_w_index + 3][0].lower().strip()
                                )
                    if self.index_valid(first_w_index + 4, blob.tags):
                        if blob.tags[last_w_index + 4][1] in [
                            "RB",
                            "RBR",
                            "RBS",
                            "JJ",
                            "JJR",
                            "JJS",
                        ] and blob.tags[first_w_index + 3][0].lower().strip() in [
                            "and"
                        ]:
                            if self.index_valid(first_w_index + 4, blob.tags):

                                after_words.append(
                                        blob.tags[last_w_index + 4][0].lower().strip()
                            )



                    if self.index_valid(first_w_index + 2, blob.tags):
                        if blob.tags[last_w_index + 2][0].lower() in ["very", "really"]:

                            if self.index_valid(first_w_index + 3, blob.tags):
                                after_words.append(
                                    blob.tags[last_w_index + 3][0].lower().strip()
                                )
                    attr = " ".join(after_words)

                    if str(a.strip()) in list(aspect_opinion_dict.keys()):

                        aspect_opinion_dict[a] = " ".join(
                            [aspect_opinion_dict[a] + " " + attr]
                        ).strip()
                    else:
                        aspect_opinion_dict[str(a).strip()] = attr
            else:
                pass

    def aspect_generator(self):

        self.candidate_aspects_dict = {}
        self.aspect_opinion_dict = {}
        aspects = []
        no_reviews=len(self.lines)
        for line in tqdm(self.lines,desc="Collecting Aspects And Opinions From {} Reviews".format(no_reviews)):

            blob = TextBlob(line)

            nouns_1 = [aspects.append(n) for n, t in blob.tags if t in ["NN", "NNS", "NNP", "NNPS"] ]
            candidate_aspects = [n for n, t in blob.tags if t in ["NN", "NNS", "NNP", "NNPS"] ]

            self._aspect_list(line, candidate_aspects, self.aspect_opinion_dict)
        keys = list(self.aspect_opinion_dict.keys())
        for a in keys:

            if a in self.stop_words:
                self.aspect_opinion_dict.pop(a)

        final_df = pd.DataFrame(
            self.aspect_opinion_dict.items(), columns=["aspects", "opinions"]
        )
        final_df["opinions"] = final_df["opinions"].apply(lambda x: str(x).strip())
        final_df["opinion_len"] = final_df["opinions"].apply(
            lambda x: len(str(x).split())
        )
        final_df["opinion_list"] = final_df["opinions"].apply(
            lambda x: Counter(str(x).split()).most_common()
        )
        self.rslt_df = final_df.sort_values(by="opinion_len", ascending=False)
        return self.aspect_opinion_dict, self.rslt_df

    @staticmethod
    def index_valid(index_last_word, blob_tags):

        return True if (0 <= (index_last_word) < len(blob_tags)) else False

    def showing_aspect(self):
        try:
            aspect_opinion_dict = self.aspect_opinion_dict

        except AttributeError:
            self.aspect_generator()
        aspect_name = list(self.rslt_df["aspects"][:9])
        aspect_opinion = list(self.rslt_df["opinion_list"][:9])
        fig = make_subplots(
            rows=3,
            cols=3,
            start_cell="top-left",

            subplot_titles=aspect_name,
        )
        cols = [1, 2, 3, 1, 2, 3, 1, 2, 3]
        rows = [1, 1, 1, 2, 2, 2, 3, 3, 3]

        for s in range(9):
            try:
                op = [w[0] for w in aspect_opinion[s][:5]][::-1]
            except:
                if len(aspect_opinion[s]) > 0:

                        op=[w[0] for w in aspect_opinion[s][:len(aspect_opinion[s])]][::-1]
            try:

                num = [w[1] for w in aspect_opinion[s][:5]][::-1]
            except:

               if len(aspect_opinion[s]) > 0:
                  num=[w[1] for w in aspect_opinion[s][:len(aspect_opinion[s])]][::-1]

            fig = fig.add_trace(
                go.Bar(
                    x=num,
                    showlegend=False,
                    marker=dict(color="rgb(140, 3, 252)", colorscale="Edge"),
                    y=op,
                    orientation="h",

                ),
                row=int(rows[s]),
                col=int(cols[s]),
            )
            fig.update_layout(
                title="People are talking about.",
                height=1200,
                showlegend=False)
        self.fig_aspect = fig



import spacy
from insightsearch.aspect_analyze import *
nlp = spacy.load("en_core_web_lg")
import regex as re
import nltk
import os

if not os.path.exists(str(Path(__file__).parent) +'/saved/corpora/wordnet'):
	nltk.download('wordnet', download_dir=str(Path(__file__).parent) +'/saved')
else:
     nltk.data.path.append(str(Path(__file__).parent) +'/saved/corpora/wordnet')
from nltk.stem import WordNetLemmatizer

class DataframeClean(Aspectanalyze):
    def __init__(self, file, column_name, date_column,vader):
        self.file = file
        self.column_name = column_name
        self.date_column = date_column
        self.vader=vader
        self.wnl = WordNetLemmatizer()

        self.common_titles = [
            "mr",
            "mrs",
            "ms",
            "miss",
            "dr",
            "herr",
            "monsieur",
            "hr",
            "frau",
            "a v m",
            "admiraal",
            "admiral",
            "alderman",
            "alhaji",
            "ambassador",
            "baron",
            "barones",
            "brig",
            "brigadier",
            "brother",
            "canon",
            "capt",
            "captain",
            "cardinal",
            "cdr",
            "chief",
            "cik",
            "cmdr",
            "col",
            "colonel",
            "commandant",
            "commander",
            "commissioner",
            "commodore",
            "comte",
            "comtessa",
            "congressman",
            "conseiller",
            "consul",
            "conte",
            "contessa",
            "corporal",
            "councillor",
            "count",
            "countess",
            "air cdre",
            "air commodore",
            "air marshal",
            "air vice marshal",
            "brig gen",
            "brig general",
            "brigadier general",
            "crown prince",
            "crown princess",
            "dame",
            "datin",
            "dato",
            "datuk",
            "datuk seri",
            "deacon",
            "deaconess",
            "dean",
            "dhr",
            "dipl ing",
            "doctor",
            "dott",
            "dott sa",
            "dr",
            "dr ing",
            "dra",
            "drs",
            "embajador",
            "embajadora",
            "en",
            "encik",
            "eng",
            "eur ing",
            "exma sra",
            "exmo sr",
            "f o",
            "father",
            "first lieutient",
            "first officer",
            "flt lieut",
            "flying officer",
            "fr",
            "frau",
            "fraulein",
            "fru",
            "gen",
            "generaal",
            "general",
            "governor",
            "graaf",
            "gravin",
            "group captain",
            "grp capt",
            "h e dr",
            "h h",
            "h m",
            "h r h",
            "hajah",
            "haji",
            "hajim",
            "her highness",
            "her majesty",
            "herr",
            "high chief",
            "his highness",
            "his holiness",
            "his majesty",
            "hon",
            "hr",
            "hra",
            "ing",
            "ir",
            "jonkheer",
            "judge",
            "justice",
            "khun ying",
            "kolonel",
            "lady",
            "lcda",
            "lic",
            "lieut",
            "lieut cdr",
            "lieut col",
            "lieut gen",
            "lord",
            "madame",
            "mademoiselle",
            "maj gen",
            "major",
            "master",
            "mevrouw",
            "miss",
            "mlle",
            "mme",
            "monsieur",
            "monsignor",
            "mstr",
            "nti",
            "pastor",
            "president",
            "prince",
            "princess",
            "princesse",
            "prinses",
            "prof",
            "prof dr",
            "prof sir",
            "professor",
            "puan",
            "puan sri",
            "rabbi",
            "rear admiral",
            "rev",
            "rev canon",
            "rev dr",
            "rev mother",
            "reverend",
            "rva",
            "senator",
            "sergeant",
            "sheikh",
            "sheikha",
            "sig",
            "sig na",
            "sig ra",
            "sir",
            "sister",
            "sqn ldr",
            "sr",
            "sr d",
            "sra",
            "srta",
            "sultan",
            "tan sri",
            "tan sri dato",
            "tengku",
            "teuku",
            "than puying",
            "the hon dr",
            "the hon justice",
            "the hon miss",
            "the hon mr",
            "the hon mrs",
            "the hon ms",
            "the hon sir",
            "the very rev",
            "toh puan",
            "tun",
            "vice admiral",
            "viscount",
            "viscountess",
            "wg cdr",
        ]

    @staticmethod
    def stop_words_removal(x, common_titles,wnl):


        # listing person names
        people = [w.text for w in nlp(x).ents if w.label_ == "PERSON"]
        stopwords = common_titles + people 
        text_lz= (" ").join([wnl.lemmatize(w) for w in x.lower().split()])
        text_wp = (" ").join([w for w in text_lz.lower().split() if w not in stopwords])
        text_wp = re.sub(r"[^\w\s]", "", text_wp).lower()

        return text_wp

    def _clean(self):
        # removing empty rows directly from the dataframe
        clean_df = self.file.copy().dropna()

        clean_df["reviewText"] = (
            clean_df["reviewText"]
            .apply(lambda x: self.stop_words_removal(x, self.common_titles,self.wnl))
            .dropna()
        )
     
        if self.date_column is not None:

                clean_df[self.date_column] = pd.to_datetime(
                    clean_df[self.date_column], format="%m %d, %Y"
                )
                clean_df = clean_df.assign(
                    year=clean_df["reviewTime"].dt.year,
                    month=clean_df["reviewTime"].dt.month,
                    day=clean_df["reviewTime"].dt.day,
              )

        Aspectanalyze.__init__(self, clean_df, self.column_name)
        Sentimentanalyze.__init__(self, clean_df, self.column_name, self.date_column,self.vader)
        return clean_df



from distutils.core import setup
import pathlib
readme_path=(pathlib.Path(__file__).parent / "README.md").read_text()
setup(
  name = 'insightsearch',         # How you named your package folder (MyLib)
  packages = ['insightsearch'],   # Chose the same as "name"
  version = '1.4',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  long_description=readme_path,
  long_description_content_type="text/markdown",
  description = 'Insightsearch will show hidden sentiments, aspects and their opinions from your text data. It will show what people are talking about your product/service',   # Give a short description about your library
  author = 'Vivek alex',                   # Type in your name
  author_email = 'vivekalexktr@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/vivekalex61/insightsearch',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/vivekalex61/insightsearch/archive/refs/tags/v1.4.tar.gz',  
  keywords = ['NLP', 'INSIGHT', 'REVIEW ANALYZER','REVIEW'],   # Keywords that define your package best
  project_urls={
         
          'Source Code': 'https://github.com/vivekalex61/insightsearch',
        
      },
  install_requires=[
          'transformers',
          'spacy',
          'pandas',
      'nltk','plotly','textblob','vaderSentiment'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)
